@@ -15,6 +15,10 @@ import pyglinet.glinet_api as api_helper
 import pathlib
 import pickle
 from typing import Union, List, Dict
+import shutil
+
+log = logging.getLogger(__name__)
+
 
 
 class GlInet:
@@ -60,7 +64,7 @@ class GlInet:
         if cache_folder is None:
             self._cache_folder = pathlib.Path.home()
             self._cache_folder = os.path.join(self._cache_folder, ".python-glinet")
-            logging.info(f"Creating folder {self._cache_folder} if not exist")
+            log.info(f"Creating folder {self._cache_folder} if not exist")
             pathlib.Path(self._cache_folder).mkdir(exist_ok=True)
         else:
             if os.path.exists(cache_folder):
@@ -78,13 +82,14 @@ class GlInet:
         self._lock = threading.Lock()
         self._verify_ssl_certificate = verify_ssl_certificate
         if self._verify_ssl_certificate is False:
-            logging.warning("You disabled ssl certificate validation. Further warning messages will be deactivated.")
+            log.warning("You disabled ssl certificate validation. Further warning messages will be deactivated.")
             warnings.filterwarnings('ignore', message='Unverified HTTPS request')
         self._cached_login_data = None
         self._login_cache_path = os.path.join(self._cache_folder, "login.pkl")
         self._api_reference_cache_path = os.path.join(self._cache_folder, "api_reference.pkl")
         self._api_reference_url = api_reference_url
         self._api_description = self.__load_api_description(update_api_reference_cache)
+        self._api = None
         self._keep_alive_interrupt_event = threading.Event()
 
     def __del__(self):
@@ -155,7 +160,8 @@ class GlInet:
             elif error_["code"] == -32602:
                 raise exceptions.WrongParametersError(f"Wrong params in request, error output: {error_}")
             elif error_["code"] == -32601:
-                raise exceptions.MethodNotFoundError(f"Wrong method {req.get('method', None)} in request, error output: {error_}")
+                raise exceptions.MethodNotFoundError(
+                    f"Wrong method {req.get('method', None)} in request, error output: {error_}")
             else:
                 raise ConnectionError(resp.json())
         if resp.status_code != 200:
@@ -234,18 +240,18 @@ class GlInet:
         resp = self.request("challenge", {"username": self._username})
         return resp.result
 
-    def login(self):
+    def login(self) -> "GlInet":
         """
         Login and start background thread for keep_alive is configured. If password was set in constructor,
         cached values will be ignored. If password was not set (default) in :meth:`~pyglinet.GlInet`, you will be asked to enter the password
         the first time this function is called. If login was successful, the password hash is cashed.
 
-        :return: True
+        :return: GlInet
         """
 
         if self.is_alive():
-            logging.info("Already logged in, nothing to do.")
-            return True
+            log.info("Already logged in, nothing to do.")
+            return self
 
         challenge = self.__challenge_login()
         if self._password is None:
@@ -263,7 +269,7 @@ class GlInet:
                                           "hash": login_hash})
             self._sid = resp.result.sid
         except exceptions.AccessDeniedError:
-            logging.warning("Could not login with current credentials, deleting cached credentials.")
+            log.warning("Could not login with current credentials, deleting cached credentials.")
             self._cached_login_data = None
             self._sid = None
             os.remove(self._login_cache_path)
@@ -272,7 +278,7 @@ class GlInet:
         # start keep alive thread
         if self._keep_alive:
             self._start_keep_alive_thread()
-        return True
+        return self
 
     @decorators.login_required
     def _start_keep_alive_thread(self):
@@ -283,7 +289,7 @@ class GlInet:
         :return:
         """
         if self._thread is None or not self._thread.is_alive():
-            logging.debug("Starting background keep alive thread.")
+            log.debug("Starting background keep alive thread.")
             self._keep_alive_interrupt_event.clear()
             self._thread = threading.Thread(target=self.__keep_alive)
             self._thread.start()
@@ -295,7 +301,7 @@ class GlInet:
         Stop keep alive thread
         """
         if hasattr(self, "_thread") and self._thread and self._thread.is_alive():
-            logging.info(f"Shutting down background thread. This will take max {self._keep_alive_intervall} seconds.")
+            log.info(f"Shutting down background thread. This will take max {self._keep_alive_intervall} seconds.")
             self._keep_alive_interrupt_event.set()
             self._thread.join()
 
@@ -335,7 +341,7 @@ class GlInet:
                 try:
                     loaded_data = pickle.load(f)
                 except:
-                    logging.warning(f"Something went wrong loading file {file}")
+                    log.warning(f"Something went wrong loading file {file}")
         return loaded_data
 
     def __dump_to_file(self, obj, file):
@@ -347,6 +353,9 @@ class GlInet:
 
         :return: None
         """
+        if not pathlib.Path(file).parent.exists():
+            pathlib.Path(file).parent.mkdir(exist_ok=True)
+
         with open(file, "wb") as f:
             pickle.dump(obj, f)
 
@@ -359,14 +368,26 @@ class GlInet:
 
         :return: None
         """
-        logging.info(f"Starting keep alive thread at intvervall {self._keep_alive_intervall}")
+        log.info(f"Starting keep alive thread at intvervall {self._keep_alive_intervall}")
         while self._keep_alive and not self._keep_alive_interrupt_event.is_set():
-            logging.debug(f"keep alive with intervall {self._keep_alive_intervall}")
+            log.debug(f"keep alive with intervall {self._keep_alive_intervall}")
             if not self.is_alive():
-                logging.warning("client disconnected, trying to login again..")
+                log.warning("client disconnected, trying to login again..")
                 self.login()
             self._keep_alive_interrupt_event.wait(self._keep_alive_intervall)
-        logging.info("Keep alive halted.")
+        log.info("Keep alive halted.")
+
+    def flush_cache(self) -> None:
+        """
+        Deletes the folder containing persisted login and api description as well as cached login data.
+        It will NOT invalidate or reload api data. To achieve that see :meth:`~pyglinet.GlInet.get_api_client`
+
+        :return: None
+        """
+        if os.path.exists(self._cache_folder):
+            shutil.rmtree(self._cache_folder)
+        self._cached_login_data = None
+        log.info(f"Login cache cleared and folder {self._cache_folder} deleted")
 
     def is_alive(self) -> bool:
         """
@@ -427,12 +448,13 @@ class GlInet:
         """
         api_description = None
         if update or not os.path.exists(self._api_reference_cache_path):
-            logging.info(f"Loading api description from {self._api_reference_url}")
+            log.info(f"Loading api description from {self._api_reference_url}")
             resp = requests.get(self._api_reference_url)
             api_description = resp.json()["data"]
             api_description = {utils.sanitize_string(i["module_name"][0]): i for i in api_description}
+            pathlib.Path(self._cache_folder).mkdir(exist_ok=True)
             with open(self._api_reference_cache_path, "wb") as f:
-                logging.info(f"Updating cache file {self._api_reference_cache_path}")
+                log.info(f"Updating cache file {self._api_reference_cache_path}")
                 pickle.dump(api_description, f)
         else:
             with open(self._api_reference_cache_path, "rb") as f:
@@ -441,13 +463,26 @@ class GlInet:
         return api_description
 
     @decorators.login_required
-    def get_api_client(self) -> api_helper.GlInetApi:
+    def get_api_client(self, update_description=False) -> api_helper.GlInetApi:
         """
         Create GlInetApi object client to access api functions
 
-        :return: api client
+        :param update_description: if True, api description is updated from web
+        :return: GlInetApi
         """
-        if self._api_description:
-            return api_helper.GlInetApi(self._api_description, self)
-        else:
-            raise exceptions.NoApiDescriptionError("Api description is not loaded.")
+        if not self._api_description or update_description:
+            self._api_description = self.__load_api_description(True)
+            self._api = api_helper.GlInetApi(self._api_description, self)
+        if not self._api:
+            self._api = api_helper.GlInetApi(self._api_description, self)
+
+        return self._api
+
+    @property
+    def api(self) -> api_helper.GlInetApi:
+        """
+        Method gives access to the autogenerated api functions. See also :meth:`~pyglinet.GlInet.get_api_client`
+
+        :return: GlInetApi
+        """
+        return self.get_api_client()
